@@ -24,6 +24,7 @@ mod ignore;
 mod lexer;
 mod mode;
 mod operators;
+mod output;
 mod parser;
 mod query;
 mod searcher;
@@ -373,6 +374,7 @@ Functions:
         BIN                         Returns binary representation of an integer value
         HEX                         Returns hexadecimal representation of an integer value
         OCT                         Returns octal representation of an integer value
+        POWER | POW                 Raise the value to the specified power
         CONTAINS                    Returns true, if file contains string, false if not
         COALESCE                    Returns first nonempty expression value
         CONCAT                      Returns concatenated string of expression values
